import sqlite3
from abc import ABC
from typing import Any, Dict

from .table_info import TableField, TableInfo
from .executor import SqliteExecutor
from ..data_source import DataSource


class SqliteDataSource(DataSource, ABC):
    def __init__(self, conf):
        self.src = conf['src']
        self.models: Dict[str, Any] = {}
        self.executor = SqliteExecutor(self)

    def get_connection(self):
        return sqlite3.connect(self.src)

    def get_data_structure(self, table):
        if table in self.models:
            return self.models[table]

        conn = self.get_connection()
        c = conn.cursor()
        try:
            result = c.execute(f'PRAGMA table_info({table})')
            rows = result.fetchall()
            table_fields = []
            for row in rows:
                table_field = TableField(cid=row[0],
                                         name=row[1],
                                         type_=row[2],
                                         notnull=row[3],
                                         dflt_value=row[4],
                                         pk=row[5])
                table_fields.append(table_field)

            table_info = TableInfo(table=table, table_fields=table_fields)
            self.models[table] = table_info.parse_model()
            return self.models[table]
        finally:
            c.close()
            conn.close()

    def get_executor(self):
        return self.executor

    def get_type(self):
        return 'sqlite'



from ..utils.str_utils import snake_case


def parametrized(decorator):
    def layer(*args, **kwargs):
        def repl(cls):
            return decorator(cls, *args, **kwargs)

        return repl

    return layer


@parametrized
def entity(cls, table: str = None, dynamic=False, ignore: list[str] = None):
    def columns(exclude: list[str] = None):
        cols = [key for key in cls.model_fields if
                (exclude is None or key not in exclude) and (ignore is None or key not in ignore)]
        return cols

    def table_name():
        if table is not None:
            return table
        return snake_case(cls.__name__)

    setattr(cls, 'columns', columns)
    setattr(cls, 'table_name', table_name)
    setattr(cls, 'dynamic', dynamic)
    return cls



import abc


class DataSource(metaclass=abc.ABCMeta):
    @abc.abstractmethod
    def get_connection(self):
        pass

    @abc.abstractmethod
    def get_data_structure(self, table):
        pass

    @abc.abstractmethod
    def get_executor(self):
        pass

    @abc.abstractmethod
    def get_type(self):
        pass


class DataSourceManager:
    data_sources = {}

    @staticmethod
    def register(name, data_source: DataSource):
        DataSourceManager.data_sources[name] = data_source

    @staticmethod
    def get(name):
        return DataSourceManager.data_sources[name]



from .config import Configuration
from .db.query_wrapper import QueryWrapper
from .db.update_wrapper import UpdateWrapper
from .db.insert_wrapper import InsertWrapper
from .db.wrapper import Wrapper
from .cache import LRUCache
from loguru import logger


class Seal:

    def __init__(self):
        self._configuration = Configuration()
        self._initialized = False
        self._data_sources = {}
        self._default_data_source = None
        self._logical_delete = None
        self._default_cache: LRUCache = LRUCache(512)

    def init(self, config_path):
        self._configuration.load(config_path)
        self._initialized = True

        logger.add(self.get_config('seal', 'loguru', 'path'),
                   rotation=self.get_config('seal', 'loguru', 'rotation'),
                   retention=self.get_config('seal', 'loguru', 'retention'),
                   level=self.get_config('seal', 'loguru', 'level'))

        data_source_config = self.get_config('seal', 'data_source')
        for data_source_name, data_source_conf in data_source_config.items():
            data_source = self.get_config('seal', 'data_source', data_source_name)
            if 'mysql' == data_source.get('type'):
                from .db.mysql.mysql_data_source import MysqlDataSource
                self._data_sources[data_source_name] = MysqlDataSource(data_source_conf)
            elif 'sqlite' == data_source.get('type'):
                from .db.sqlite.sqlite_data_source import SqliteDataSource
                self._data_sources[data_source_name] = SqliteDataSource(data_source_conf)
            else:
                raise ValueError(f'不支持的数据源类型: {data_source.get("type")}')

            if data_source_conf.get('default', False):
                self._default_data_source = self._data_sources[data_source_name]

        if len(self._data_sources.values()) == 1 and self._default_data_source is None:
            self._default_data_source = list(self._data_sources.values())[0]
        if self._default_data_source is None:
            raise ValueError('未配置默认数据源')
        logger.info(f'初始化 seal({self}) 成功')
        return self

    def data_source(self, data_source_name):
        if not self._initialized:
            raise ValueError('Seal 未初始化')
        return self._data_sources.get(data_source_name)

    def get_config(self, *keys):
        if not self._initialized:
            raise ValueError('Seal 未初始化')
        return self._configuration.get_conf(*keys)

    def q(self, table):
        return QueryWrapper(table, self._default_data_source, self._logical_delete)

    def query_wrapper(self, table):
        return QueryWrapper(table, self._default_data_source, self._logical_delete)

    def u(self, table):
        return UpdateWrapper(table, self._default_data_source, self._logical_delete)

    def update_wrapper(self, table):
        return UpdateWrapper(table, self._default_data_source, self._logical_delete)

    def i(self, table):
        return InsertWrapper(table, self._default_data_source)

    def insert_wrapper(self, table):
        return InsertWrapper(table, self._default_data_source)

    def wrapper(self):
        return Wrapper(self._logical_delete)

    def raw(self, sql, args=()):
        return self._default_data_source.get_executor().raw(sql, args)

    def enable_logical_delete(self, logical_delete):
        self._logical_delete = logical_delete
        return self

    def model(self, model_name):
        return self._default_data_source.get_data_structure(model_name)

    def default_cache(self) -> LRUCache:
        return self._default_cache



class Condition:
    def __init__(self, field, value, operator='='):
        self.field = field
        self.value = value
        self.operator = operator

    def parse(self):
        return f'{self.field} {self.operator} ?', self.value


class ConditionTree:
    def __init__(self, logic='and'):
        self.conditions = []
        self.logic = logic

    def or_(self):
        self.logic = 'or'
        return self

    def add_condition(self, condition: Condition):
        self.conditions.append(condition)
        return self

    def add_tree(self, condition_tree):
        self.conditions.append(condition_tree)
        return self

    def parse(self):
        if len(self.conditions) == 0:
            return None
        args = []
        exps = []
        for condition in self.conditions:
            if isinstance(condition, ConditionTree):
                exp, arg = condition.parse()
                exps.append(f'({exp})')
                args.extend(arg)
            else:
                exp, arg = condition.parse()
                exps.append(exp)
                args.append(arg)
        return f' {self.logic} '.join(exps), tuple(args)



def group(list_, key_func: callable, value_func: callable = None):
    """
    Group a list of dictionaries by a key.
    """
    grouped = {}
    for item in list_:
        key = key_func(item)
        if key not in grouped:
            grouped[key] = []
        if value_func:
            grouped[key].append(value_func(item))
        else:
            grouped[key].append(item)
    return grouped



import asyncio
import time
import re
from functools import wraps
import jwt
from fastapi import FastAPI, Request, Response, HTTPException, Depends
from starlette.middleware.cors import CORSMiddleware
from ..context import WebContext
from ..model import Response as ResponseModel
from ..exception import BusinessException
from .. import seal
from loguru import logger


async def verify_token(request: Request = Request):
    if request.method == 'OPTIONS':
        return None
    path = request.url.path
    urls = seal.get_config('seal', 'authorization', 'excludes')
    for url in urls:
        # * -> [a-zA-Z0-9_\-]*, ** -> .*
        url = url.replace('*', '[a-zA-Z0-9_\-]*').replace('**', '.*')
        if re.match(url, path):
            return None
    try:
        payload = jwt.decode(request.headers['Authorization'],
                             seal.get_config('seal', 'authorization', 'jwt_key'),
                             algorithms=["HS256"])
        WebContext().set({'uid': payload['uid']})
    except Exception as e:
        raise HTTPException(status_code=401, detail=f"Token is invalid: {e}")


app = FastAPI(dependencies=[Depends(verify_token)])


@app.middleware("http")
async def calc_time(request: Request, call_next):
    start_time = time.time()
    response = await call_next(request)
    process_time = time.time() - start_time
    response.headers["X-Process-Time"] = str(process_time)
    return response


app.add_middleware(
    CORSMiddleware,
    allow_origins=["*"],
    allow_credentials=True,
    allow_methods=["*"],
    allow_headers=["*"],
    expose_headers=["*"]
)


@app.exception_handler(HTTPException)
async def http_exception_handler(request: Request, exc: HTTPException):
    return Response(status_code=exc.status_code, content=exc.detail)


def response_body(func):
    @wraps(func)
    async def wrapper(*args, **kwargs):
        try:
            task = asyncio.create_task(func(*args, **kwargs))
            result = await task
            return ResponseModel.build(result).success()
        except BusinessException as e:
            logger.exception(e)
            return ResponseModel.build().error(message=e.message, code=e.code)
        except Exception as e:
            logger.exception(e)
            return ResponseModel.build().error(message=str(e))

    return wrapper


def get(path: str):
    def decorator(func):
        return app.get(path, response_model=ResponseModel)(response_body(func))

    return decorator


def post(path: str):
    def decorator(func):
        return app.post(path, response_model=ResponseModel)(response_body(func))

    return decorator


def delete(path: str):
    def decorator(func):
        return app.delete(path, response_model=ResponseModel)(response_body(func))

    return decorator


def put(path: str):
    def decorator(func):
        return app.put(path, response_model=ResponseModel)(response_body(func))

    return decorator



from dataclasses import fields
from .sql_builder import build_insert, build_insert_bulk, build_insert_iterator


class InsertWrapper:
    def __init__(self, table, data_source):
        self.table = table
        self.data_source = data_source
        self.param_type = data_source.get_data_structure(table)
        self.insert_fields = []

    def insert(self, data, **options):
        if data is None:
            raise ValueError('null data')

        if isinstance(data, dict):
            keys = data.keys()
            self.insert_fields = [f.name for f in fields(self.param_type) if
                                  f.name != 'id' and f.name in keys]
        else:
            self.insert_fields = [f.name for f in fields(self.param_type) if
                                  f.name != 'id' and getattr(data, f.name) is not None]

        if 'duplicated_key_update' in options:
            sql, args = build_insert(self, data, options['duplicated_key_update'])
        else:
            sql, args = build_insert(self, data)
        return self.data_source.get_executor().insert(sql, args)

    def insert_bulk(self, data_list, **options):
        if data_list is None or len(data_list) == 0:
            raise ValueError('null data')

        if isinstance(data_list[0], dict):
            keys = data_list[0].keys()
            self.insert_fields = [f.name for f in fields(self.param_type) if
                                  f.name != 'id' and f.name in keys]
        else:
            self.insert_fields = [f.name for f in fields(self.param_type) if
                                  f.name != 'id' and getattr(data_list[0], f.name) is not None]

        if 'duplicated_key_update' in options:
            sql, args = build_insert_bulk(self, data_list, duplicated_key_update=options['duplicated_key_update'])
        elif 'duplicated_key_ignore' in options:
            sql, args = build_insert_bulk(self, data_list, duplicated_key_ignore=options['duplicated_key_ignore'])
        else:
            sql, args = build_insert_bulk(self, data_list)
        return self.data_source.get_executor().insert_bulk(sql, args)

    def insert_iterator(self, data_list, **options):
        if data_list is None or len(data_list) == 0:
            raise ValueError('null data')

        if isinstance(data_list[0], dict):
            keys = data_list[0].keys()
            self.insert_fields = [f.name for f in fields(self.param_type) if
                                  f.name != 'id' and f.name in keys]
        else:
            self.insert_fields = [f.name for f in fields(self.param_type) if
                                  f.name != 'id' and getattr(data_list[0], f.name) is not None]

        data_iterator = build_insert_iterator(self, data_list, **options)
        return self.data_source.get_executor().insert_interator(data_iterator)



from typing import Tuple, Callable, Any, Generator

from loguru import logger


def build_select(query_wrapper) -> (str, tuple):
    sql = f'SELECT {",".join(query_wrapper.field_list)} FROM {query_wrapper.table}'
    args = ()
    if len(query_wrapper.condition_tree.conditions) > 0:
        exp, args = query_wrapper.condition_tree.parse()
        sql += ' WHERE ' + exp
    if query_wrapper.order_by is not None:
        sql += f' ORDER BY {",".join(query_wrapper.order_by)}'
    if query_wrapper.limit is not None:
        sql += f' LIMIT {query_wrapper.limit}'
    if query_wrapper.offset is not None:
        sql += f' OFFSET {query_wrapper.offset}'

    logger.debug(f'#### sql: {sql}')
    logger.debug(f'#### args: {args}')
    return sql, args


def build_count(query_wrapper) -> (str, tuple):
    sql = f'SELECT COUNT(1) FROM {query_wrapper.table}'
    args = ()
    if len(query_wrapper.condition_tree.conditions) > 0:
        exp, args = query_wrapper.condition_tree.parse()
        sql += ' WHERE ' + exp

    logger.debug(f'#### sql: {sql}')
    logger.debug(f'#### args: {args}')
    return sql, args


def build_update(update_wrapper) -> (str, tuple):
    sql = f'UPDATE {update_wrapper.table} SET {",".join([f"{k}=?" for k in update_wrapper.update_fields.keys()])}'
    args = tuple(update_wrapper.update_fields.values())
    if len(update_wrapper.condition_tree.conditions) > 0:
        exp, args2 = update_wrapper.condition_tree.parse()
        sql += ' WHERE ' + exp
        args += args2
    logger.debug(f'#### sql: {sql}')
    logger.debug(f'#### args: {args}')
    return sql, args


def build_delete(update_wrapper) -> (str, tuple):
    sql = f'DELETE FROM {update_wrapper.table}'
    args = ()
    if len(update_wrapper.condition_tree.conditions) > 0:
        exp, args = update_wrapper.condition_tree.parse()
        sql += ' WHERE ' + exp
    logger.debug(f'#### sql: {sql}')
    logger.debug(f'#### args: {args}')
    return sql, args


def build_insert(insert_wrapper, data, duplicated_key_update=False, duplicated_key_ignore=False) -> (str, tuple):
    keys = None
    if isinstance(data, dict):
        keys = data.keys()

    sql = f'INSERT {"OR IGNORE" if duplicated_key_ignore else ""} INTO {insert_wrapper.table} ({",".join([field for field in insert_wrapper.insert_fields if keys is None or field in keys])}) VALUES ({",".join(["?" for field in insert_wrapper.insert_fields if keys is None or field in keys])})'

    if isinstance(data, dict):
        args = tuple([data[field] for field in insert_wrapper.insert_fields if field in keys])
    else:
        args = tuple([getattr(data, field) for field in insert_wrapper.insert_fields])

    if duplicated_key_update:
        sql += f' ON DUPLICATE KEY UPDATE {",".join([f"{field}=?" for field in insert_wrapper.insert_fields if keys is None or field in keys])}'
        args = args * 2
    logger.debug(f'#### sql: {sql}')
    logger.debug(f'#### args: {args}')
    return sql, args


def build_insert_bulk(insert_wrapper, data_list, duplicated_key_update=False, duplicated_key_ignore=False) -> (
        str, list[tuple]):
    keys = None
    data = data_list[0]
    if isinstance(data, dict):
        keys = data.keys()

    sql = f'INSERT {"OR IGNORE" if duplicated_key_ignore else ""} INTO {insert_wrapper.table} ({",".join([field for field in insert_wrapper.insert_fields if keys is None or field in keys])}) VALUES ({",".join(["?" for field in insert_wrapper.insert_fields if keys is None or field in keys])})'

    if duplicated_key_update:
        sql += f' ON DUPLICATE KEY UPDATE {",".join([f"{field}=?" for field in insert_wrapper.insert_fields])}'

    if duplicated_key_update:
        if isinstance(data, dict):
            args = [tuple([data[field] for field in insert_wrapper.insert_fields if field in keys]) * 2 for data in
                    data_list]
        else:
            args = [tuple([getattr(data, field) for field in insert_wrapper.insert_fields]) * 2 for data in data_list]
    else:
        if isinstance(data, dict):
            args = [tuple([data[field] for field in insert_wrapper.insert_fields if field in keys]) for data
                    in data_list]
        else:
            args = [tuple([getattr(data, field) for field in insert_wrapper.insert_fields]) for data in data_list]
    logger.debug(f'#### sql: {sql}')
    logger.debug(f'#### args: {args}')
    return sql, args


def build_insert_iterator(insert_wrapper, data_list, duplicated_key_update=False, duplicated_key_ignore=False):
    keys = None
    if isinstance(data_list[0], dict):
        keys = data_list[0].keys()

    sql = f'INSERT {"OR IGNORE" if duplicated_key_ignore else ""} INTO {insert_wrapper.table} ({",".join([field for field in insert_wrapper.insert_fields if keys is None or field in keys])}) VALUES ({",".join(["?" for field in insert_wrapper.insert_fields if keys is None or field in keys])})'

    if duplicated_key_update:
        sql += f' ON DUPLICATE KEY UPDATE {",".join([f"{field}=?" for field in insert_wrapper.insert_fields])}'

    def data_iterator(callback):
        for data in data_list:
            if isinstance(data, dict):
                args = tuple([data[field] for field in insert_wrapper.insert_fields if field in keys])
            else:
                args = tuple([getattr(data, field) for field in insert_wrapper.insert_fields])
            logger.debug(f'#### args: {args}')
            callback(sql, args)

    logger.debug(f'#### sql: {sql}')
    return data_iterator



class BusinessException(Exception):

    def __init__(self, code=1, message='业务异常'):
        self.code = code
        self.message = message

    def __str__(self):
        return f"{self.code} - {self.message}"



import yaml


class Configuration:

    def __init__(self):
        self.config_dict = {}

    def load(self, path: str):
        with open(path, 'r') as f:
            self.config_dict = yaml.load(f, Loader=yaml.FullLoader)

    def get_conf(self, *keys):
        conf = self.config_dict
        for key in keys:
            conf = conf.get(key)
        return conf



from contextvars import ContextVar
from ..wrapper import singleton

web_ctx = ContextVar('web', default={})


@singleton
class WebContext:
    def __init__(self):
        self.ctx = web_ctx

    def get(self):
        return self.ctx.get()

    def set(self, value: dict):
        self.ctx.set(value)

    def uid(self):
        var = self.get()
        if 'uid' not in var:
            return None
        return var['uid']



from loguru import logger
from ..data_source import DataSource
from ..executor import Executor


class SqliteExecutor(Executor):

    def __init__(self, data_source: DataSource):
        super().__init__(data_source)

    def find(self, sql, args, select_fields, result_type, **options):
        connection = self.data_source.get_connection()
        cursor = connection.cursor()
        try:
            result = cursor.execute(sql, args)
            if result is None:
                return None

            row = cursor.fetchone()
            if row is None:
                return None

            if options.get('to_dict', False) is True:
                return {field: row[i] for i, field in enumerate(select_fields)}
            return result_type(**{field: row[i] for i, field in enumerate(select_fields)})
        except Exception as e:
            logger.exception(e)
            raise e
        finally:
            cursor.close()
            connection.close()

    def find_list(self, sql, args, select_fields, result_type, **options):
        connection = self.data_source.get_connection()
        cursor = connection.cursor()
        try:
            result = cursor.execute(sql, args)
            if result is None:
                return None

            rows = cursor.fetchall()
            if rows is None:
                return None

            if options.get('to_dict', False) is True:
                return [{field: row[i] for i, field in enumerate(select_fields)} for row in rows]
            return [result_type(**{field: row[i] for i, field in enumerate(select_fields)}) for row in rows]
        except Exception as e:
            logger.exception(e)
            raise e
        finally:
            cursor.close()
            connection.close()

    def count(self, sql, args):
        connection = self.data_source.get_connection()
        cursor = connection.cursor()
        try:
            result = cursor.execute(sql, args)
            if result is None:
                return None

            row = cursor.fetchone()
            if row is None:
                return None
            return row[0]
        except Exception as e:
            logger.exception(e)
            raise e
        finally:
            cursor.close()
            connection.close()

    def update(self, sql, args):
        connection = self.data_source.get_connection()
        cursor = connection.cursor()
        try:
            result = cursor.execute(sql, args)
            connection.commit()
            if result is None:
                return None
            return cursor.rowcount
        except Exception as e:
            logger.exception(e)
            raise e
        finally:
            cursor.close()
            connection.close()

    def insert(self, sql, args):
        connection = self.data_source.get_connection()
        cursor = connection.cursor()
        try:
            result = cursor.execute(sql, args)
            connection.commit()
            if result is None:
                return None
            return cursor.lastrowid
        except Exception as e:
            logger.exception(e)
            raise e
        finally:
            cursor.close()
            connection.close()

    def insert_bulk(self, sql, args):
        connection = self.data_source.get_connection()
        cursor = connection.cursor()
        try:
            result = cursor.executemany(sql, args)
            connection.commit()
            if result is None:
                return None
            return cursor.rowcount
        except Exception as e:
            logger.exception(e)
            raise e
        finally:
            cursor.close()
            connection.close()

    def insert_interator(self, data_iterator):
        connection = self.data_source.get_connection()
        cursor = connection.cursor()
        try:
            data_iterator(lambda sql, args: cursor.execute(sql, args))
            connection.commit()
            return cursor.rowcount
        except Exception as e:
            logger.exception(e)
            raise e
        finally:
            cursor.close()
            connection.close()



from .wrapper import Wrapper
from .data_source import DataSource
from .sql_builder import build_select, build_count
from dataclasses import fields


class QueryWrapper(Wrapper):
    def __init__(self, table: str, data_source: DataSource, logical_delete=None):
        super().__init__(logical_delete=logical_delete)
        self.table = table
        self.data_source = data_source
        self.result_type = data_source.get_data_structure(table)
        self.limit = None
        self.offset = None
        self.order_by = None
        self.field_list = []
        self.ignore_fields = []

    def select(self, *field_list):
        self.field_list = field_list
        return self

    def ignore(self, *field_list):
        self.ignore_fields = field_list
        return self

    def sort(self, *order_by):
        self.order_by = order_by
        return self

    def limit(self, limit: int):
        self.limit = limit
        return self

    def offset(self, offset: int):
        self.offset = offset
        return self

    def find(self, **options):
        self._handle_logical_delete(**options)

        sql, args = self._build_select()
        return self.data_source.get_executor().find(sql, args, self.field_list, self.result_type, **options)

    def find_list(self, **options):
        self._handle_logical_delete(**options)

        sql, args = self._build_select()
        return self.data_source.get_executor().find_list(sql, args, self.field_list, self.result_type, **options)

    def find_page(self, page: int, page_size: int, **options):
        self._handle_logical_delete(**options)

        self.limit = page_size
        self.offset = (page - 1) * page_size
        sql, args = self._build_select()
        records = self.data_source.get_executor().find_list(sql, args, self.field_list, self.result_type, **options)
        count = self.count()
        return records, count

    def count(self):
        sql, args = build_count(self)
        return self.data_source.get_executor().count(sql, args)

    def _build_select(self):
        if len(self.field_list) == 0:
            self.field_list = [field.name for field in fields(self.result_type) if field.name not in self.ignore_fields]
        return build_select(self)



from .data_source import DataSource
from loguru import logger


class Executor:

    def __init__(self, data_source: DataSource):
        self.data_source = data_source

    def raw(self, sql, args=()):
        cursor = self.data_source.get_connection().cursor()
        try:
            result = cursor.execute(sql, args)
            if result is None:
                return None

            return cursor.fetchall()
        except Exception as e:
            logger.exception(e)
            raise e
        finally:
            cursor.close()



from datetime import datetime


class BaseEntity:
    id: int | None = None
    create_by: int | None = None
    update_by: int | None = None
    create_at: datetime | None = None
    update_at: datetime | None = None
    deleted: int | None = 0



import jwt
from datetime import datetime, timedelta
from .. import seal


def generate_token(**payloads):
    try:
        payloads["exp"] = datetime.now() + timedelta(seconds=seal.get_config('seal', 'authorization', 'expire'))
        token = jwt.encode(payloads, seal.get_config('seal', 'authorization', 'jwt_key'), algorithm="HS256")
    except Exception as e:
        raise Exception(f"Token generation failed: {e}")
    return token



from dataclasses import make_dataclass, field
from typing import Any


class TableField:

    def __init__(self, cid, name, type_, notnull, dflt_value, pk):
        self.cid = cid
        self.name = name
        self.type_ = type_
        self.notnull = notnull
        self.dflt_value = dflt_value
        self.pk = pk


class TableInfo:

    def __init__(self, table, table_fields: list[TableField]):
        self.table = table
        self.table_fields = table_fields
        self.model = None

    def parse_model(self):
        if self.model is None:
            self.model = make_dataclass(self.table,
                                        [(table_field.name, Any, field(default=None)) for table_field in
                                         self.table_fields])
        return self.model



from .wrapper import Wrapper
from .sql_builder import build_update, build_delete
from dataclasses import fields


class UpdateWrapper(Wrapper):

    def __init__(self, table, data_source, logical_delete=None):
        super().__init__(logical_delete=logical_delete)
        self.table = table
        self.data_source = data_source
        self.update_fields = {}

    def set(self, field, value):
        self.update_fields[field] = value
        return self

    def set_model(self, entity):
        if isinstance(entity, dict):
            for field in fields(self.data_source.get_data_structure(self.table)):
                if field.name != 'id' and field.name in entity:
                    self.update_fields[field.name] = entity[field.name]
            return self

        for field in fields(entity):
            if field.name != 'id':
                continue
            self.update_fields[field.name] = getattr(entity, field.name)
        return self

    def update(self, **options):
        if len(self.condition_tree.conditions) == 0:
            raise ValueError('不支持全量更新')

        self._handle_logical_delete(**options)

        sql, args = build_update(self)
        return self.data_source.get_executor().update(sql, args)

    def delete(self, **options):
        if len(self.condition_tree.conditions) == 0:
            raise ValueError('不支持全量更新')

        self._handle_logical_delete(**options)

        if options.get('logical_delete', False):
            self.set(options['logical_delete'], 1)
            return self.update(**options)
        else:
            sql, args = build_delete(self)
            return self.data_source.get_executor().update(sql, args)



import time
import pymysql
from pymysql.cursors import DictCursor
from pymysql.connections import Connection as PyMySQLConnection


class ConnectionPool:
    def __init__(self, conf):
        self.conf = conf

        self._connections: list[DelegateConnection] = []
        self._min_connections = conf['pool']['min_connections']
        self._max_connections = conf['pool']['max_connections']

        for _ in range(self._min_connections):
            mysql_connection: PyMySQLConnection = pymysql.connect(host=conf['host'],
                                                                  user=conf['user'],
                                                                  password=conf['password'],
                                                                  database=conf['database'],
                                                                  cursorclass=DictCursor)
            self._connections.append(DelegateConnection(mysql_connection, self))

    def new_connection(self):
        return pymysql.connect(host=self.conf['host'],
                               user=self.conf['user'],
                               password=self.conf['password'],
                               database=self.conf['database'],
                               cursorclass=DictCursor)

    def get_connection(self, timeout=None):
        start_time = time.time()

        connection = self.occupy()
        if connection is not None:
            return connection

        if len(self._connections) < self._max_connections:
            mysql_connection = self.new_connection()
            connection = DelegateConnection(mysql_connection, self)
            connection.occupy()
            self._connections.append(connection)
            return connection

        if timeout is not None:
            raise Exception('No available connection')

        while True:
            time.sleep(0.1)

            if timeout is not None and time.time() - start_time > timeout:
                raise Exception('No available connection')

            connection = self.occupy()
            if connection is not None:
                return connection

    def occupy(self):
        for connection in self._connections:
            if connection.status == 'idle':
                connection.occupy()
                return connection
        return None

    def release(self, delegate_connection):
        if len(self._connections) > self._min_connections:
            delegate_connection.connection.close()
            self._connections.remove(delegate_connection)
        else:
            delegate_connection.status = 'idle'


class DelegateConnection:
    def __init__(self, connection: PyMySQLConnection, pool: ConnectionPool, create_time=time.time()):
        self.connection: PyMySQLConnection = connection
        self.pool: ConnectionPool = pool
        self.status = 'idle'
        self.create_time = create_time

    def close(self):
        self.pool.release(self)

    def cursor(self):
        return self.connection.cursor()

    def occupy(self):
        self.status = 'occupied'

    def commit(self):
        self.connection.commit()

    def rollback(self):
        self.connection.rollback()

    def begin(self):
        self.connection.ping(reconnect=True)
        self.connection.begin()



from abc import ABC
from typing import Dict, Any

from .table_info import TableField, TableInfo
from ..data_source import DataSource
from .connection_pool import ConnectionPool
from .executor import MysqlExecutor


class MysqlDataSource(DataSource, ABC):
    def __init__(self, conf):
        self.connection_pool = ConnectionPool(conf)
        self.models: Dict[str, Any] = {}
        self.executor = MysqlExecutor(self)

    def get_connection(self):
        return self.connection_pool.get_connection(10)

    def get_data_structure(self, table):
        if table in self.models:
            return self.models[table]

        conn = self.get_connection()
        c = conn.cursor()
        try:
            c.execute(f'show columns from {table}')
            rows = c.fetchall()
            table_fields = []
            for row in rows:
                table_field = TableField(field_=row['Field'],
                                         type_=row['Type'],
                                         null_=row['Null'],
                                         key_=row['Key'],
                                         default_=row['Default'],
                                         extra=row['Extra'])
                table_fields.append(table_field)

            table_info = TableInfo(table=table, table_fields=table_fields)
            self.models[table] = table_info.parse_model()
            return self.models[table]
        finally:
            c.close()
            conn.close()

    def get_executor(self):
        return self.executor

    def get_type(self):
        return 'mysql'



from .condition import ConditionTree, Condition
from typing import Self


class Wrapper:
    def __init__(self, logical_delete=None):
        self.condition_tree = ConditionTree()
        self.logical_delete = logical_delete

    def eq(self, field, value):
        self.condition_tree.add_condition(Condition(field, value))
        return self

    def ne(self, field, value):
        self.condition_tree.add_condition(Condition(field, value, '!='))
        return self

    def gt(self, field, value):
        self.condition_tree.add_condition(Condition(field, value, '>'))
        return self

    def ge(self, field, value):
        self.condition_tree.add_condition(Condition(field, value, '>='))
        return self

    def lt(self, field, value):
        self.condition_tree.add_condition(Condition(field, value, '<'))
        return self

    def le(self, field, value):
        self.condition_tree.add_condition(Condition(field, value, '<='))
        return self

    def in_(self, field, value):
        self.condition_tree.add_condition(Condition(field, value, 'in'))
        return self

    def l_like(self, field, value):
        self.condition_tree.add_condition(Condition(field, f'%{value}', 'like'))
        return self

    def r_like(self, field, value):
        self.condition_tree.add_condition(Condition(field, f'{value}%', 'like'))
        return self

    def like(self, field, value):
        self.condition_tree.add_condition(Condition(field, f'%{value}%', 'like'))
        return self

    def or_(self, wrapper: Self):
        wrapper.condition_tree.or_()
        self.condition_tree.add_tree(wrapper.condition_tree)
        return self

    def _handle_logical_delete(self, **options):
        if 'logical_delete' in options:
            self.eq(options['logical_delete'], 0)
            return

        if self.logical_delete is not None:
            self.eq(self.logical_delete, 0)
